# -*- coding: utf-8 -*-
"""Interface definitions."""

# zope imports
from zope import schema
from zope.interface import Interface

# local imports
from plone.mls.core.i18n import _


class IMLSSettings(Interface):
    """Global Propertyshelf MLS settings.

    This describes records stored in the configuration registry and obtainable
    via plone.registry.
    """

    mls_key = schema.TextLine(
        default=u'',
        required=True,
        title=_(u'MLS API Key'),
    )

    mls_site = schema.TextLine(
        default=u'',
        required=True,
        title=_(u'MLS URL'),
    )

    agency_id = schema.TextLine(
        default=u'',
        required=True,
        title=_(u'Agency ID'),
    )


class IPossibleLocalMLSSettings(Interface):
    """Marker interface for possible local MLS settings."""


class ILocalMLSSettings(Interface):
    """Marker interface for activated local MLS settings."""



# -*- coding: utf-8 -*-
"""Migration steps for plone.mls.listing."""

# zope imports
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from zope.component import getUtility


PROFILE_ID = 'profile-plone.mls.core:default'


def migrate_to_1001(context):
    """Migrate from 1000 to 1001.

    * Activate portal actions.
    * Register JS resources.
    """
    site = getUtility(IPloneSiteRoot)
    setup = getToolByName(site, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'actions')
    setup.runImportStepFromProfile(PROFILE_ID, 'jsregistry')



# -*- coding: utf-8 -*-
"""plone.mls.core API."""

# python imports
import logging

# zope imports
from persistent.interfaces import IPersistent
from Acquisition import aq_parent, aq_inner
from Products.CMFPlone.interfaces import IPloneSiteRoot
from plone import api
from plone.registry.interfaces import IRegistry
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.globalrequest import getRequest

# local imports
from plone.mls.core.browser.localconfig import CONFIGURATION_KEY
from plone.mls.core.interfaces import (
    ILocalMLSSettings,
    IMLSSettings,
)


logger = logging.getLogger('plone.mls.core')


def _local_settings(context):
    """Get local MLS settings."""
    settings = None
    if not context or not IPersistent.providedBy(context):
        request = getRequest()
        parents = request.get('PARENTS')
        if len(parents) > 0:
            context = parents[0]
        else:
            context = api.portal.get()
    obj = context
    while (
            not IPloneSiteRoot.providedBy(obj) and
            not ILocalMLSSettings.providedBy(obj)):
        parent = aq_parent(aq_inner(obj))
        if parent is None:
            return
        obj = parent
    if ILocalMLSSettings.providedBy(obj):
        annotations = IAnnotations(obj)
        settings = annotations.get(
            CONFIGURATION_KEY, annotations.setdefault(CONFIGURATION_KEY, {}))
    return settings


def get_settings(context=None):
    """Get the MLS settings."""
    if context is None:
        context = api.portal.get()

    local_settings = _local_settings(context)
    if local_settings:
        logger.debug('Returning local MLS settings.')
        return local_settings

    # Get the global configuration.
    settings = {}
    registry = getUtility(IRegistry)
    if registry is not None:
        try:
            registry_settings = registry.forInterface(IMLSSettings)
        except:
            logger.warning('Global MLS settings not available.')
        else:
            settings = dict([
                (a, getattr(registry_settings, a)) for a in
                registry_settings.__schema__]
            )
            logger.debug('Returning global MLS settings.')
    return settings



# -*- coding: utf-8 -*-
"""Custom Batch Provider for listing results."""

# zope imports
from Products.CMFPlone.PloneBatch import Batch
try:
    from Products.CMFPlone.PloneBatch import LazyNextBatch, LazyPrevBatch
    USE_OLD_BATCHING = True
except ImportError:
    USE_OLD_BATCHING = False


if USE_OLD_BATCHING:
    class LazyListingPrevBatch(LazyPrevBatch):
        """Previous listing result batch."""

        def __of__(self, parent):
            return ListingBatch(
                parent._sequence,
                parent._size,
                parent.first - parent._size + parent.overlap,
                0,
                parent.orphan,
                parent.overlap,
                batch_data=parent.batch_data,
            )

    class LazyListingNextBatch(LazyNextBatch):
        """Next listing result batch."""

        def __of__(self, parent):
            if parent.end >= (parent.last + parent.size):
                return None
            return ListingBatch(
                parent._sequence,
                parent._size,
                parent.end - parent.overlap,
                0,
                parent.orphan,
                parent.overlap,
                batch_data=parent.batch_data,
            )

    class ListingBatch(Batch):
        """Listing result batch."""
        __allow_access_to_unprotected_subobjects__ = 1

        previous = LazyListingPrevBatch()
        next = LazyListingNextBatch()

        def __init__(self, sequence, size, start=0, end=0, orphan=0, overlap=0,
                     pagerange=7, quantumleap=0, b_start_str='b_start',
                     batch_data=None):
            self.batch_data = batch_data
            if self.batch_data is not None:
                length = 0
                if sequence is not None:
                    length = len(sequence)
                self.sequence_length = self.batch_data.get('results', length)

            if sequence is None:
                sequence = []

            super(ListingBatch, self).__init__(
                sequence, size, start, end, orphan, overlap, pagerange,
                quantumleap, b_start_str,
            )

        def __getitem__(self, index):
            if index >= self.length:
                raise IndexError(index)
            return self._sequence[index]

else:
    class ListingBatch(Batch):
        """Listing result batch."""

        def __init__(self, sequence, size, start=0, end=0, orphan=0, overlap=0,
                     pagerange=7, quantumleap=0, b_start_str='b_start',
                     batch_data=None):
            self.batch_data = batch_data

            if sequence is None:
                sequence = []

            super(ListingBatch, self).__init__(
                sequence, size, start, end, orphan, overlap, pagerange,
                quantumleap, b_start_str,
            )

        @property
        def sequence_length(self):
            """Effective length of sequence."""
            if self.batch_data is not None:
                length = getattr(self, 'pagesize', 0)
                return self.batch_data.get('results', length)
            return super(ListingBatch, self).sequence_length

        def __getitem__(self, index):
            if index >= self.length:
                raise IndexError(index)
            return self._sequence[index]

        @property
        def next(self):
            """Next batch page."""
            if USE_OLD_BATCHING:
                return LazyListingNextBatch()

            if self.end >= (self.last + self.pagesize):
                return None
            return ListingBatch(
                self._sequence, self._size, self.end - self.overlap, 0,
                self.orphan, self.overlap, batch_data=self.batch_data,
            )

        @property
        def previous(self):
            """Previous batch page."""
            if USE_OLD_BATCHING:
                return LazyListingPrevBatch()

            if not self.first:
                return None
            return ListingBatch(
                self._sequence, self._size,
                self.first - self._size + self.overlap, 0, self.orphan,
                self.overlap, batch_data=self.batch_data,
            )



# -*- coding: utf-8 -*-
"""MLS Settings Control Panel."""

# zope imports
from plone.app.registry.browser import controlpanel

# local imports
from plone.mls.core.i18n import _
from plone.mls.core.interfaces import IMLSSettings


class MLSSettingsEditForm(controlpanel.RegistryEditForm):
    """Global/default MLS Settings Form."""

    schema = IMLSSettings
    label = _(u'Propertyshelf MLS Settings')
    description = _(
        u'This MLS configuration will be used as the default for this '
        u'Plone site. You can add more MLS configurations by activating '
        u'the local MLS settings on any content item within the site.'
    )

    def updateFields(self):
        super(MLSSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(MLSSettingsEditForm, self).updateWidgets()


class MLSSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    """MLS Settings Control Panel."""

    form = MLSSettingsEditForm
